package org.cebs.corep.excelToXbrl.action;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.rmi.RemoteException;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.upload.FormFile;
import org.cebs.corep.excelToXbrl.mapper.ExcelSheetReader;
import org.cebs.corep.forms.CorepForm;
import org.cebs.corep.xbrl.action.CorepAction;
import org.cebs.corep.xbrl.exceptions.CorepBusinessException;
import org.cebs.corep.xbrl.instanceGenerator.InstanceGenerator;
import org.cebs.corep.xbrl.log.LogAction;
import org.cebs.corep.xbrl.util.CorepClientContext;
import org.cebs.corep.xbrl.util.DocumentValidator;
import org.cebs.corep.xbrl.util.JdomConverter;

/**
 * This class is responsible for the upload of the excel test files.
 * <br />
 */
public class ExcelFileTransformationAction extends CorepAction
{
  /**
   * This method reads the test file and starts the transformation process.
   * Then the xbrl instance document is shown in a new window of the browser.
   * <br />
   *
   * @param  mapping     ActionMapping 
   * @param  actionForm  ActionForm 
   * @param  request     HttpServletRequest 
   * @param  response    HttpServletResponse 
   * @throws RemoteException
   */
  public void performAction(ActionMapping mapping, ActionForm actionForm, HttpServletRequest request, HttpServletResponse response) throws CorepBusinessException, RemoteException
  {
    String methodName = "performAction";
    log(LogAction.METHOD_BEGIN, this.getClass().getName(), methodName);

    CorepForm form = (CorepForm)actionForm;

    // initialise an object of PrintWriter
    PrintWriter out = null;

    // output to the browser in XML, encoding "UTF-8"
    response.setContentType("text/xml; charset=UTF-8");

    // not to open in the browser
    String filename = null;
    filename = StringUtils.left(form.getUploadedFile().getFileName(), form.getUploadedFile().getFileName().length() - 4);

    response.setHeader("Content-disposition", "attachment;filename=" + filename + ".xml");
    try
    {
      out = response.getWriter();
    }
    catch (IOException e)
    {
    }

    // transforms the jdom document object into a string and sends the output to the browser
    out.print(JdomConverter.convertDocumentToString(form.getJdomDocument(), (CorepClientContext)getClientContext()));

    log(LogAction.METHOD_END, this.getClass().getName(), methodName);
  }

  /**
   * Validiert den eingegebenen Zeitraum deren Dateien angezeigt werden sollen.
   * <br />
   *
   * @param  errors      ActionErrors Ein ActionErrors-Objekt zum Einstellen von Fehlermeldungen.
   * @param  actionForm  ActionForm Optional ActionForm fuer diesen Request.
   * @return ActionErrors
   */
  public ActionErrors validate(ActionErrors errors, ActionForm actionForm)
  {
    String methodName = "validate";
    log(LogAction.METHOD_BEGIN, this.getClass().getName(), methodName);

    CorepForm form = (CorepForm)actionForm;

    form.setResult(null);

    // in case of the trial of a transformation of an erroneous file
    try
    {
      FormFile uploadedFile = form.getUploadedFile();

      // the test file should end with "xls"
      if (form.getUploadedFile().getFileName().endsWith("xls"))
      {
        // transforms the file object into an object of type InputStream
        InputStream uploadInStream = uploadedFile.getInputStream();

        // the data is read byte to byte into another object of type InputStream
        InputStream in = ExcelSheetReader.upload(uploadInStream, (CorepClientContext)getClientContext());

        // put the data into a map which keys are the table names and the values are 
        // a arrayList of Excel rows (HSSFRow) 
        HashMap workbookContent = ExcelSheetReader.readWorkbookContent(in, (CorepClientContext)getClientContext());

        // generates an XBRL instance document on basis of the map that contains the content of the excel sheets
        form.setJdomDocument(InstanceGenerator.generateXbrlInstance(workbookContent, (CorepClientContext)getClientContext()));

        // validation according the validation level
        if (form.getValidationLevel().equalsIgnoreCase("no"))
        {
          int wellformed = DocumentValidator.isWellFormed(JdomConverter.convertDocumentToByteArray(form.getJdomDocument(), (CorepClientContext)getClientContext()), (CorepClientContext)getClientContext());

          if (wellformed < DocumentValidator.WELL_FORMED_AND_WELL_ENCODED)
          {
            errors.add("corepBusinessException", new ActionError("ErrorReport_NotWellformed"));
          }
        }
        else if (form.getValidationLevel().equalsIgnoreCase("schema") || form.getValidationLevel().equalsIgnoreCase("xbrl"))
        {
          // validates the instance document
          form.setErrorMessage(DocumentValidator.validateDocument(form.getJdomDocument(), (CorepClientContext)getClientContext()));
          // im Fehlerfall errorMessage ausgeben
          if (StringUtils.isNotBlank(form.getErrorMessage()))
          {
            errors.add("corepBusinessException", new ActionError("ErrorReport_SchemaValidationFailed", form.getUploadedFile().getFileName()));
          }
        }
      }
      else
      { // sends an error message to the browser
        errors.add("corepBusinessException", new ActionError("ErrorReport_NoUpload"));
      }
    }
    catch (FileNotFoundException e)
    {
      log(LogAction.DEBUG, this.getClass().getName(), "performAction", "The file could not be found.", e);
      errors.add("corepBusinessException", new ActionError("ErrorReport_FileNotFound"));
    }
    catch (IOException e)
    {
      log(LogAction.DEBUG, this.getClass().getName(), "performAction", "File not readable.", e);
      errors.add("corepBusinessException", new ActionError("ErrorReport_ReadFile"));
    }
    catch (Throwable t)
    {
      log(LogAction.TECHNICAL_ERROR, this.getClass().getName(), "performAction", "File could not be transformed into XBRL.", t);
      errors.add("corepBusinessException", new ActionError("ErrorReport_TransformFile"));
    }
    log(LogAction.METHOD_END, this.getClass().getName(), methodName);
    return errors;
  }
}
