package org.cebs.corep.excelToXbrl.mapper;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Properties;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.cebs.corep.excelToXbrl.constants.XbrlConstantsInterface;
import org.cebs.corep.xbrl.instanceGenerator.ElementCreator;
import org.cebs.corep.xbrl.log.CorepLogger;
import org.cebs.corep.xbrl.log.LogAction;
import org.cebs.corep.xbrl.util.CorepClientContext;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * This class builds an xml jdom tree out of a list of HSSFRow objects.
 */
public class RowsToXbrlMapper extends ElementCreator implements XbrlConstantsInterface
{
  private static final String CLASSNAME = "RowsToXbrlMapper";

  /**
   * This method builds depending on the template name an xml jdom tree
   * on basis of the rows of each excel sheet.
   * @param tableName			String
   * @param rows				ArrayList
   * @param parentElement		Element
   * @param referenceNumber		String
   * @param cc					CorepClientContext
   */
  public static void createXbrlTemplate(String tableName, ArrayList rows, Element parentElement, String referenceNumber, String instantDate, int sheetsCounter, CorepClientContext cc)
  {
    CorepLogger.log(LogAction.METHOD_BEGIN, CLASSNAME, "createXbrlTemplate for Template" + tableName, cc);

    // OPR templates  
    if (tableName.endsWith(T_OPR))
      new TemplatesOPRtoXbrlMapper().transform_OPR(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_OPR_DETAILS))
      new TemplatesOPRtoXbrlMapper().transform_OPR_DETAILS(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_OPR_LOSS_DETAILS))
      new TemplatesOPRtoXbrlMapper().transform_OPR_LOSS(rows, parentElement, referenceNumber, sheetsCounter, instantDate);

    // MR templates
    else if (tableName.endsWith(T_MKR_IM))
      new TemplatesMKRIMtoXbrlMapper().transform_MKR_IM(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_MKR_DI))
      new TemplatesMKRIMtoXbrlMapper().transform_MKR_Daily_Day(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_MKR_SA_TDI))
      new TemplatesMKRSAtoXbrlMapper().transform_MKR_SA_TDI(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_MKR_SA_EQU))
      new TemplatesMKRSAtoXbrlMapper().transform_MKR_SA_EQU(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_MKR_SA_COM))
      new TemplatesMKRSAtoXbrlMapper().transform_MKR_SA_COM(rows, parentElement, referenceNumber, sheetsCounter, instantDate);
    else if (tableName.endsWith(T_MKR_SA_FX))
      new TemplatesMKRSAtoXbrlMapper().transform_MKR_SA_FX(rows, parentElement, referenceNumber, sheetsCounter, instantDate);

    CorepLogger.log(LogAction.METHOD_END, CLASSNAME, "createXbrlTemplate for Template" + tableName, cc);
  }

  /**
   * Detects the reference number of the coreptest file.i.
   * 
   * @param workbookContent	HashMap
   * @param cc		CorepClientContext
   * @return
   */
  public static String getReferenceNumber(HashMap workbookContent, CorepClientContext cc)
  {
    CorepLogger.log(LogAction.METHOD_BEGIN, CLASSNAME, "getReferenceNumber", cc);

    // gets the overview page
    ArrayList rows = (ArrayList)workbookContent.get(OVERVIEW_PAGE);

    // reads the row with the reference number
    HSSFRow row = (HSSFRow)rows.get(2);
    String referenceNumber = getCellValue(row, 1);

    CorepLogger.log(LogAction.METHOD_END, CLASSNAME, "getReferenceNumber", cc);

    return referenceNumber;
  }

  /**
  	* Detects the reference number of the coreptest file.i.
  	* 
  	* @param workbookContent	HashMap
  	* @param cc		CorepClientContext
  	* @return
  	*/
  public static String getInstantDate(HashMap workbookContent, CorepClientContext cc)
  {
    CorepLogger.log(LogAction.METHOD_BEGIN, CLASSNAME, "getInstantDate", cc);

    // gets the overview page
    ArrayList rows = (ArrayList)workbookContent.get(OVERVIEW_PAGE);

    // reads the row with the date
    HSSFRow row = (HSSFRow)rows.get(3);

    Date date = null;
    String instantDate = null;
    if (row.getCell((short)1) != null)
    {
      try
      {
        date = row.getCell((short)1).getDateCellValue();
        SimpleDateFormat mDateFormatInput = new SimpleDateFormat("yyyy-MM-dd", Locale.GERMAN);
        instantDate = mDateFormatInput.format(date);
      }
      catch (Exception e)
      { // transformation in a string
        instantDate = getCellValue(row, 1);
      }
    }
    if (StringUtils.isBlank(instantDate))
      instantDate = DEFAULT_INSTANT;

    CorepLogger.log(LogAction.METHOD_END, CLASSNAME, "getInstantDate", cc);

    return instantDate;
  }

  /**
   * This method checks the content type of the cell and returns for numeric
   * and string values a string.
   * 
   * @param row			HSSFRow
   * @param cellnum		cellnum
   * @return			String
   */
  protected static String getCellValue(HSSFRow row, int cellnum)
  {
    String value = null;

    HSSFCell cell = row.getCell((short)cellnum);

    if (cell != null)
    {
      // numeric cell type
      if (cell.getCellType() == 0)
      {
        Double doubleValue = new Double(cell.getNumericCellValue());
        value = doubleValue.toString();
      }
      // string cell type    
      else if (cell.getCellType() == 1)
      {
        value = cell.getStringCellValue();
      }
    }
    return value;
  }

  /**
   * This method returns the content of the cell as string 
   * without decimals for the monetaryItemType elements.
   * @param row			HSSFRow
   * @param cellnum		cellnum
   * @return			String
   */
  protected static String getMonetaryValue(HSSFRow row, int cellnum)
  {
    String value = null;
    if (row.getCell((short)cellnum) != null)
    {
      try
      {
        Double doubleValue = new Double(row.getCell((short)cellnum).getNumericCellValue());
        long longValue = Math.round(doubleValue.doubleValue());
        value = Long.toString(longValue);
      }
      catch (RuntimeException e)
      {
        value = null;
      }
    }
    return value;
  }

  /**
   * This method transforms the content of the cell in a date value
   * and back into a string.
   * 
   * @param row			HSSFRow
   * @param cellnum		cellnum
   * @return			String
   */
  protected static String getDateValue(HSSFRow row, int cellnum)
  {
    Date date = null;
    String value = null;
    if (row.getCell((short)cellnum) != null)
    {
      try
      {
        date = row.getCell((short)cellnum).getDateCellValue();
        SimpleDateFormat mDateFormatInput = new SimpleDateFormat("yyyy-MM-dd", Locale.GERMAN);
        value = mDateFormatInput.format(date);
      }
      catch (Exception e)
      { // transformation in a string
        value = getCellValue(row, cellnum);
      }
    }
    return value;
  }

  /**
   * This method transforms the content of the cell into a integer
   * value and back in a string.
   * 
   * @param row			HSSFRow
   * @param cellnum		cellnum
   * @return			String
   */
  protected static String getIntegerValue(HSSFRow row, int cellnum)
  {
    String value = null;

    HSSFCell cell = row.getCell((short)cellnum);

    if (cell != null)
    {
      // numeric cell type without decimals
      if (cell.getCellType() == 0)
      {
        Double doubleValue = new Double(cell.getNumericCellValue());
        long longValue = Math.round(doubleValue.doubleValue());
        value = Long.toString(longValue);
      }
      // string cell type, transformation without decimals   
      else if (cell.getCellType() == 1)
      {
        try
        {
          Double doubleValue = new Double(cell.getStringCellValue());
          long longValue = Math.round(doubleValue.doubleValue());
          value = Long.toString(longValue);
        }
        // transformation in a string
        catch (NumberFormatException e)
        {
          value = getCellValue(row, cellnum);
        }
      }
    }
    return value;
  }

  /**
   * This method transforms the content of the cell into a integer
   * value and back in a string with a negative value.
   * 
   * @param row			HSSFRow
   * @param cellnum		cellnum
   * @return			String
   */
  protected static String getNonPositiveIntegerValue(HSSFRow row, int cellnum)
  {
    String value = getIntegerValue(row, cellnum);

    if (StringUtils.isNotBlank(value) && !StringUtils.right(value, 1).equalsIgnoreCase("-"))
      value = "-" + value;

    return value;
  }

  /**
   * Adds a namespace declaration to the root element when it is missing.
   * 
   * @param ns
   * @param rootElement
   */
  protected static void addNamespaceDeclaration(Namespace ns, Element rootElement)
  {
    List namespaceList = rootElement.getAdditionalNamespaces();
    if (!namespaceList.contains(ns))
      rootElement.addNamespaceDeclaration(ns);
  }
  
	/**
	 * Checks if the row has content and return a boolean.
	 * A column can be disclosed from this validation.
	 * @param tuple
	 * @param row
	 * @param parentElement
	 */
	public boolean checkRowContent(HSSFRow row, int columnBegin, int[] withoutColumn)
	{
		boolean hasRowContent = false;
		// list of rows
		HashMap rowContent = new HashMap();
		short i;

		for (i = (short)columnBegin; i < row.getLastCellNum(); i++)
		{
			HSSFCell cell = (HSSFCell)row.getCell(i);
			if (cell != null && !ArrayUtils.contains(withoutColumn, i))
				rowContent.put(new Integer(i), new Integer(cell.getCellType()));
		}
		if (!rowContent.isEmpty())
			hasRowContent = true;

		return hasRowContent;
	}

  /**
   * Checks if the row has content and return a boolean.
   * @param tuple
   * @param row
   * @param parentElement
   */
  public boolean checkRowContent(HSSFRow row, int columnBegin)
  {
    boolean hasRowContent = false;
    // list of rows
    ArrayList rowContent = new ArrayList();

    for (short i = (short)columnBegin; i < row.getLastCellNum(); i++)
    {
      HSSFCell cell = (HSSFCell)row.getCell(i);
      if (cell != null)
        rowContent.add(new Integer(cell.getCellType()));
    }
    if (!rowContent.isEmpty())
      hasRowContent = true;

    return hasRowContent;
  }

  /**
   * Checks if the row has content and return a boolean.
   * @param row
   */
  public boolean checkRowContent(ArrayList row)
  {
    boolean hasRowContent = false;
    // list of rows
    ArrayList rowContent = new ArrayList();

    for (int i = 1; i < row.size(); i++)
    {
      String cell = (String)row.get(i);
      if (StringUtils.isNotBlank(cell))
        rowContent.add(cell);
    }
    if (!rowContent.isEmpty())
      hasRowContent = true;

    return hasRowContent;
  }

  /**
   * Create a context element for the dimensional templates.
   * 
   * @param row
   * @param cxtCounter
   * @param props
   * @param referenceNumber
   * @return
   */
  public Element createDimensionContext(int rowColumnNumber, int cxtCounter, Properties props, String referenceNumber, String instantDate)
  {
    Namespace nsContext = Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL);
    Namespace nsDimsContext = Namespace.getNamespace(XBRLDI_NS, XBRLDI_NS_URL);

    // context name is detected out of the properties object
    String contextPart = (String)props.get(CONTEXT_DIMENSIONS);

    // creates a general context element
    Element context = createGeneralContextElement(contextPart + cxtCounter, referenceNumber, instantDate);

    // creates a scenario element with child elements
    Element scenario = new Element(SCENARIO, nsContext);

    // creates in explicit dimension element
    Element dimMemRef = new Element(EXPLICIT_DIM_MEM_REF, nsDimsContext);
    dimMemRef.setAttribute(ATTRIBUTE_TYPE, ATTRIBUTE_CONTENT_TYPE, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
    // dimension name is detected ouf of the properties object 
    String dimensionName = (String)props.get(CONTEXT_DIMENSION_HREF);
    // adds attributes to the dim element
    dimMemRef.setAttribute(ATTRIBUTE_HREF, dimensionName, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
    // the dimension names are detected out of the properties object based on the row number
    String dimensionItem = (String)props.get(new Integer(rowColumnNumber));
    // when content exists the scenario element will be added to its parent
    if (StringUtils.isNotBlank(dimensionItem))
    {
      // add the context jdom tree
      dimMemRef.addContent(dimensionItem);
      scenario.addContent(dimMemRef);
      context.addContent(scenario);
    }

    return context;
  }

  /**
  	* Create a further explicit dimension reference element.
  	* 
  	* @param props
  	* @param row
  	* @param context
  	*/
  public void createFurtherExplicitDimension(Properties props, Integer rowNum, Element context)
  {
    // creates in explicit dimension element
    Element dimMemRef = new Element(EXPLICIT_DIM_MEM_REF, Namespace.getNamespace(XBRLDI_NS, XBRLDI_NS_URL));
    dimMemRef.setAttribute(ATTRIBUTE_TYPE, ATTRIBUTE_CONTENT_TYPE, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
    // dimension name is detected ouf of the properties object 
    String dimensionName = (String)props.get(CONTEXT_DIMENSION_HREF);
    // adds attributes to the dim element
    dimMemRef.setAttribute(ATTRIBUTE_HREF, dimensionName, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));
    // the dimension names are detected out of the properties object based on the row number
    String dimensionItem = (String)props.get(rowNum);
    // when content exists the scenario element will be added to its parent

    if (StringUtils.isNotBlank(dimensionItem))
    {
      // add the context jdom tree
      dimMemRef.addContent(dimensionItem);
      // adds a second dimension to the context
      Element scenario = context.getChild(SCENARIO, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));
      scenario.addContent(dimMemRef);
    }
  }

  /**
   * Create a context element for the dimensional templates.
   * 
   * @param row
   * @param cxtCounter
   * @param props
   * @param referenceNumber
   * @return
   */
  public Element createTypedDimensionElement(Element dimension)
  {
    Namespace nsDimsContext = Namespace.getNamespace(XBRLDI_NS, XBRLDI_NS_URL);

    Element typedDimMemRef = new Element(TYPED_DIM_MEM_REF, nsDimsContext);
    typedDimMemRef.setAttribute(ATTRIBUTE_TYPE, ATTRIBUTE_CONTENT_TYPE, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));

    // adds attributes to the dim element
    String dimensionId = dimension.getNamespacePrefix() + NS_DATE + ".xsd#" + dimension.getNamespacePrefix() + "_" + dimension.getName() + "Dimension";
    typedDimMemRef.setAttribute(ATTRIBUTE_HREF, dimensionId, Namespace.getNamespace(XLINK_NS, XLINK_NS_URL));

    typedDimMemRef.addContent(dimension);
    return typedDimMemRef;
  }

  /**
   * Creates the href of the explicitMember element.
   * @param ns
   * @param dimension
   * @return
   */
  public String getDimensionLink(String ns, String dimension)
  {
    //f. i. t-op-2005-12-31.xsd#t-op_BankingActivitiesDimension 
    return ns + NS_DATE + XSD + "#" + ns + "_" + dimension;
  }

  /**
  	* Returns the complete URL of the namespace of the taxonomy.
  	* @param taxonomyPrefix
  	* @return
  	*/
  protected String getNamespaceURL(String taxonomyPrefix)
  {
    return NS_URL + taxonomyPrefix + NS_DATE;
  }
}
