package org.cebs.corep.excelToXbrl.mapper;

import java.util.ArrayList;
import java.util.Properties;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.cebs.corep.excelToXbrl.constants.MKR_IM_ConstantsInterface;
import org.cebs.corep.excelToXbrl.constants.XbrlConstantsInterface;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
 * for the templates MKR_IM und MKR_IM_Daily.
 */
public class TemplatesMKRIMtoXbrlMapper extends RowsToXbrlMapper implements XbrlConstantsInterface, MKR_IM_ConstantsInterface
{
  /**
   * This method transforms the rows of the template MKR_IM in a xbrl representation.
   * 
   * @param rows			ArrayList
   * @param parentElement	Element
   */
  public void transform_MKR_IM(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace ns = Namespace.getNamespace(P_MI_NS, getNamespaceURL(P_MI_NS));

    int cxtCounter = 1;
    String context = CONTEXT_PART + sheetsCounter + "_";
    // creates a properties object that map a dimension element name to a row
    Properties props = mapDimensionToRow(context);

    int col = COLUMN_BEGIN_MKR_IM;

    // creates the partial xml tree based on the rows of the template
    for (int i = ROWS_BEGIN_MKR_IM; i < rows.size(); i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i);

      createMonetaryElement(parentElement, getMonetaryValue(row, COLUMN_BEGIN_MKR_IM), MULTIPLICATIONFACTORXAVERAGEPREVIOUS60WORKINGDAYSVAR, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, COLUMN_BEGIN_MKR_IM + 1), PREVIOUSDAYVAR, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, COLUMN_BEGIN_MKR_IM + 2), SPECIFICRISKSURCHARGE, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, COLUMN_BEGIN_MKR_IM + 3), INCREMENTALDEFAULTRISKSURCHARGE, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, COLUMN_BEGIN_MKR_IM + 4), CAPITALREQUIREMENTS, ns, context + cxtCounter);
      createIntegerElement(parentElement, getIntegerValue(row, COLUMN_BEGIN_MKR_IM + 5), NUMBEROVERSHOOTINGS, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, COLUMN_BEGIN_MKR_IM + 6), MULTIPLICATIONFACTOR, ns, context + cxtCounter);

      // create dimensional context element
      if (checkRowContent(row, col))
        parentElement.addContent(createDimensionContext(row.getRowNum(), cxtCounter, props, referenceNumber, instantDate));

      // raises the context counter    
      cxtCounter++;
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * row number.
   * @return
   */
  private Properties mapDimensionToRow(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, DIMENSIONNAMEMARKETRISKFACTORS);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 1), D_PM_NS + ":" + TRADEDDEBTINSTRUMENTS);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 3), D_PM_NS + ":" + TRADEDDEBTINSTRUMENTSGENERALRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 4), D_PM_NS + ":" + TRADEDDEBTINSTRUMENTSSPECIFICRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 5), D_PM_NS + ":" + EQUITIES);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 6), D_PM_NS + ":" + EQUITIESGENERALRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 7), D_PM_NS + ":" + EQUITIESSPECIFICRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 8), D_PM_NS + ":" + FOREIGNEXCHANGERISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 9), D_PM_NS + ":" + COMMODITYRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 10), D_PM_NS + ":" + TOTALAMOUNTGENERALRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_IM + 11), D_PM_NS + ":" + TOTALAMOUNTSPECIFICRISK);

    return props;
  }

  /**
   * This method transforms the rows of the template MKR IM Daily in a xbrl representation.
   * 
   * @param rows			ArrayList
   * @param parentElement	Element
   */
  public void transform_MKR_Daily_Day(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    String[] basicInfos = getBasicInformation(rows);

    // get namespace of the measure taxonomy
    Namespace ns = Namespace.getNamespace(P_MD_NS, getNamespaceURL(P_MD_NS));

    int cxtCounter = 1;
    String context_part = CONTEXT_DIMENSIONS + "_" + T_MD_NS + "_" + sheetsCounter + "_";
    String context = null;

    // creates the partial xml tree based on the rows of the template
    for (int i = ROWS_BEGIN_MKR_DAILY_DAY; i < rows.size() - 3; i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i);

      // creates elements if content of the cell exists
      context = context_part + cxtCounter;
      createMonetaryElement(parentElement, getMonetaryValue(row, 2), VART10, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 3), VART1, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 4), SPECIFICRISKSURCHAGE, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 5), INCREMENTALDEFAULTRISKSURCHARGE2, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 6), INTERNALVAR, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 7), INTERNALVARLIMIT, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 8), HYPOTHETICALPL, ns, context);
      createMonetaryElement(parentElement, getMonetaryValue(row, 9), ACTUAL, ns, context);

      // raises the context counter
      if (checkRowContent(row, COLUMN_BEGIN_MKR_DAILY_DAY))
      {
        // creates a general context element
        Element contextElement = createGeneralContextElement(context_part + cxtCounter, referenceNumber, instantDate);

        // creates a scenario element 
        Element scenario = new Element(SCENARIO, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));
        //	adds the additial dimension to the scenario element of the context
        Element dayElement = new Element(DAY, Namespace.getNamespace(T_MD_NS, getNamespaceURL(T_MD_NS)));
        // gets the national market value
        dayElement.addContent(new Integer(row.getRowNum() - 14).toString());

        // add the implicit dimension "day"
        scenario.addContent(createTypedDimensionElement(dayElement));
        // add the implicit dimension basic information
        scenario.addContent(createTypedDimensionElement(createBasicInformationElement(basicInfos)));

        // adds the context element to its parent
        contextElement.addContent(scenario);
        parentElement.addContent(contextElement);
        // raises the context counter
        cxtCounter++;
      }
    }
  }

  private String[] getBasicInformation(ArrayList rows)
  {
    // row with the content of the basic information
    HSSFRow row = (HSSFRow)rows.get(ROW_BEGIN_MKR_DAILY_BASIC);

    String[] basicInfo = new String[6];
		basicInfo[0] = getIntegerValue(row, 1);
		basicInfo[1] = getIntegerValue(row, 2);
		basicInfo[2] = getIntegerValue(row, 3);
		basicInfo[3] = getIntegerValue(row, 4);
    basicInfo[4] = getCellValue(row, 5);
    basicInfo[5] = getCellValue(row, 6);

    return basicInfo;
  }

  private int getIntValue(String content)
  {
    int value;
    try
    {
      value = new Integer(content).intValue();
    }
    catch (NumberFormatException e)
    {
      value = 0;
    }

    return value;
  }

  private Element createBasicInformationElement(String[] basicInfos)
  {
    // create the complexe element for the basic information
    Namespace tNs = Namespace.getNamespace(T_MD_NS, getNamespaceURL(T_MD_NS));
    Element basicInformation = new Element(BASICINFORMATION, tNs);

    if (basicInfos[0] != null)
    {
      Element instrumentCode = new Element(INSTRUMENTCODE, tNs);
      instrumentCode.setText(basicInfos[0]);
			basicInformation.addContent(instrumentCode);
    }
    if (basicInfos[1] != null)
    {
      Element equities = new Element(SPECIFICRISKEQUITIESCALCULATIONCODE, tNs);
      equities.setText(basicInfos[1]);
			basicInformation.addContent(equities);
    }
    if (basicInfos[2] != null)
    {
      Element instrument = new Element(SPECIFICRISKDEBTINSTRUMENTCALCULATIONCODE, tNs);
      instrument.setText(basicInfos[2]);
			basicInformation.addContent(instrument);
    }
    if (basicInfos[3] != null)
    {
      Element overshootings = new Element(PLCALCULATIONNUMBEROVERSHOOTINGS, tNs);
      overshootings.setText(basicInfos[3]);
			basicInformation.addContent(overshootings);
    }
    if (basicInfos[4] != null)
    {
      Element internalVal = new Element(CONFIDENCEINTERVALINTERNALVAR, tNs);
      internalVal.setText(basicInfos[4]);
			basicInformation.addContent(internalVal);
    }
    if (basicInfos[5] != null)
    {
      Element holdingPeriod = new Element(HOLDINGPERIODINTERNALVAR, tNs);
      holdingPeriod.setText(basicInfos[5]);
      basicInformation.addContent(holdingPeriod);
    }

    return basicInformation;
  }
}