package org.cebs.corep.xbrl.action;

import java.io.Serializable;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionServlet;

import de.bundesbank.util.webgui.struts.DefaultUseCaseImpl;

/**
 * Euro-Evidenz Implementierung der Klasse <code>ActionForm</code> aus dem
 * Struts-Framework. Diese Klasse dient als Grundgeruest fuer jede
 * <code>ActionForm</code>-Klasse, die in einem Use Case benoetigt wird.
 * <br />
 * Die Klasse kann auf JSP-Seiten mit Hilfe des &lt;bakism:useForm&gt;-Tags
 * eingebunden werden und stellt die entsprechenden Getter- und Setter-
 * Methoden zur Verfuegung.
 * <br />
 * Die <code>EuroActionForm</code>-Implementierungen enthalten keine
 * Verarbeitungslogik fuer die Anwendung, da diese in den jeweiligen
 * <code>EuroAction</code>-Klassen implementiert wird.
 * <br />
 *
 * @author    alf konrath
 */
public class CorepActionForm extends DefaultUseCaseImpl implements Serializable, ErrorModelInterface
{
  // speichert die bei Validierung aufgetretenen Fehler
  private ActionErrors errors;

  /** Konstruktor. Beim Instanziieren der EuroActionForm wird ein EuroSessionToken erzeugt. */
  public CorepActionForm()
  {
    errors = new ActionErrors();
  }

  /**
   * Fuege neuen Fehler hinzu
   *
   * @param  property  String Feldbezeichner
   * @param  key       String Fehlerschluessel
   */
  public void addError(String property, String key)
  {
    // Fuege neuen Fehler hinzu
    errors.add(property, new ActionError(key));
  }

  /**
   * Fuege neuen Fehler hinzu
   *
   * @param  property  String Feldbezeichner
   * @param  key       String Fehlerschluessel
   * @param  value0    String Parameter/Replacement Value
   */
  public void addError(String property, String key, String value0)
  {
    // Fuege neuen Fehler hinzu
    errors.add(property, new ActionError(key, value0));
  }

  /**
   * Fuege neuen Fehler hinzu
   *
   * @param  property  String Feldbezeichner
   * @param  key       String Fehlerschluessel
   * @param  value0    String Parameter/Replacement Value
   * @param  value1    String Parameter/Replacement Value
   */
  public void addError(String property, String key, String value0, String value1)
  {
    // Fuege neuen Fehler hinzu
    errors.add(property, new ActionError(key, value0, value1));
  }

  /**
   * Fuege neuen Fehler hinzu
   *
   * @param  property  String Feldbezeichner
   * @param  key       String Fehlerschluessel
   * @param  value0    String Parameter/Replacement Value
   * @param  value1    String Parameter/Replacement Value
   * @param  value2    String Parameter/Replacement Value
   */
  public void addError(String property, String key, String value0, String value1, String value2)
  {
    // Fuege neuen Fehler hinzu
    errors.add(property, new ActionError(key, value0, value1, value2));
  }

  /**
   * Fuege neuen Fehler hinzu
   *
   * @param  property  String Feldbezeichner
   * @param  key       String Fehlerschluessel
   * @param  value0    String Parameter/Replacement Value
   * @param  value1    String Parameter/Replacement Value
   * @param  value2    String Parameter/Replacement Value
   * @param  value3    String Parameter/Replacement Value
   */
  public void addError(String property, String key, String value0, String value1, String value2, String value3)
  {
    // Fuege neuen Fehler hinzu
    errors.add(property, new ActionError(key, value0, value1, value2, value3));
  }

  /**
   * Ueberpruefen, ob ein bestehender key in den ActionErrors existiert.
   *
   * @param  key  String
   * @return      boolean
   */
  public boolean contains(String key)
  {
    // Falls noch kein Fehler vorhanden, ist errors null, gib false zurueck
    boolean hasKey = true;
    if (errors == null || errors.size(key) == 0)
    {
      hasKey = false;
    }

    return hasKey;
  }

  /**
   * Diese Methode liefert das Servlet.
   * <br />
   *
   * @return    ActionServlet servlet
   */
  public ActionServlet getActionServlet()
  {
    ActionServlet servlet = getServlet();

    if (servlet == null)
    {
      servlet = CorepActionServlet.getInstance();
    }

    return servlet;
  }

  /**
   *  Gibt das errors Attribut der Klasse EuroActionForm zurueck.
   *
   * @return    Der errors Wert
   */
  public ActionErrors getErrors()
  {
    return errors;
  }

  /**
   *  Setzt das errors Attribut der Klasse EuroActionForm
   *
   * @param  newErrors  Der neue errors Wert
   */
  public void setErrors(ActionErrors newErrors)
  {
    errors = newErrors;
  }

}
