package org.cebs.corep.xbrl.action;


import org.apache.struts.action.ActionErrors;


/**
 * Dieses Interface definiert Fehler in der Anwendung Euro-Evidenz.
 * Die Klasse EuroActionForm implementiert das EuroErrors-Interface.
 * <br />
 *
 * @author     Ralf Gutensohn
 * <br />
 */
public interface ErrorModelInterface
{
  /**
   * Fuege neuen Fehler hinzu.
   * Der Identifier der Fehlemeldung wird aus der messages_[language].properties+
   * ausgelesen.
   *
   *
   * @param  field     String feldname (property), bei dem der Fehler aufgetreten ist
   * @param  errormsg  String Identifier fuer die Fehlermeldung
   */
  public void addError( String field, 
  						String errormsg );

  /**
   * Fuege neuen Fehler hinzu.
   * Der Identifier der Fehlemeldung wird aus der messages_[language].properties
   * ausgelesen. Der Parameter errorparam0/1/2/3 kann einen Wert enthalten mit dem eine
   * Referenz (z.B. [0]) im Wert des Identifiers fuer die Ausgabe ersetzt wird.
   *
   * @param  field        String feldname (property), bei dem der Fehler aufgetreten ist
   * @param  errormsg     String Identifier fuer die Fehlermeldung
   * @param  errorparam0  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   */
  public void addError( String field, 
  						String errormsg, 
  						String errorparam0 );

  /**
   * Fuege neuen Fehler hinzu.
   * Der Identifier der Fehlemeldung wird aus der messages_[language].properties
   * ausgelesen. Der Parameter errorparam0/1/2/3 kann einen Wert enthalten mit dem eine
   * Referenz (z.B. [0]) im Wert des Identifiers fuer die Ausgabe ersetzt wird.
   *
   * @param  field        String feldname (property), bei dem der Fehler aufgetreten ist
   * @param  errormsg     String Identifier fuer die Fehlermeldung
   * @param  errorparam0  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   * @param  errorparam1  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   */
  public void addError( String field, 
                        String errormsg, 
                        String errorparam0, 
                        String errorparam1 );

  /**
   * Fuege neuen Fehler hinzu.
   * Der Identifier der Fehlemeldung wird aus der messages_[language].properties
   * ausgelesen. Der Parameter errorparam0/1/2/3 kann einen Wert enthalten mit dem eine
   * Referenz (z.B. [0]) im Wert des Identifiers fuer die Ausgabe ersetzt wird.
   *
   * @param  field        String feldname (property), bei dem der Fehler aufgetreten ist
   * @param  errormsg     String Identifier fuer die Fehlermeldung
   * @param  errorparam0  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   * @param  errorparam1  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   * @param  errorparam2  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   */
  public void addError( String field, 
  						String errormsg, 
  						String errorparam0, 
  						String errorparam1, 
  						String errorparam2 );

  /**
   * Fuege neuen Fehler hinzu.
   * Der Identifier der Fehlemeldung wird aus der messages_[language].properties
   * ausgelesen. Der Parameter errorparam0/1/2/3 kann einen Wert enthalten mit dem eine
   * Referenz (z.B. [0]) im Wert des Identifiers fuer die Ausgabe ersetzt wird.
   *
   * @param  field        String feldname (property), bei dem der Fehler aufgetreten ist
   * @param  errormsg     String Identifier fuer die Fehlermeldung
   * @param  errorparam0  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   * @param  errorparam1  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   * @param  errorparam2  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   * @param  errorparam3  String Parameterwert zum Setzen eines Wertes innerhalb der Fehlermeldung
   */
  public void addError( String field, 
  					    String errormsg, 
  					    String errorparam0, 
  					    String errorparam1, 
  					    String errorparam2, 
  					    String errorparam3 );

  /**
   * Ueberpruefen, ob bereits ein Fehler fr ein best. feld (field) besteht
   *
   * @param  field  String
   * @return        boolean
   */
  public boolean contains( String field );

  /**
   * Gib alle Fehler zurck
   *
   * @return    ActionErrors
   */
  public ActionErrors getErrors();
}
