package org.cebs.corep.xbrl.util;

import java.io.Serializable;

/**
 * Die Klasse EuroClientContext enthaelt clientspezifische Informationen fuers Loggen.
 * Die Vaterklasse stammt aus dem LogFramework aus Stuttgart.
 * Diese Klasse erweitert den Context im wesentlichen um die Benutzer ID,
 * welche fuer das Fuellen der Datenbankspalten "UserCreated" und "UserChanged"
 * benoetigt wird. Klasse wurde weitgehend aus Bakis uebernommen.
 * Wer hat diese Klasse eigentlich erstellt? Was macht "setClientContextLogField()"? 
 */

public class CorepClientContext extends de.bundesbank.util.log.ClientContext implements Serializable, Cloneable
{
  private String userID;
  private String sessionID;

  /**
   * Leerkonstruktor, setzt einen anonymen Benutzer
   */
  public CorepClientContext()
  {
    super();
    updateRecord();
  }

  /** 
   * ueberschreibt Methode * @see java.lang.Object#toString()
   * mit der Lieferung des ClientContextLogField
   */
  public String toString()
  {
    return getClientContextLogField();
  }
  /**
   * setzt einen anonymen Benutzer 
  */
  private void updateRecord()
  {
    StringBuffer buf = new StringBuffer();
	//folgender "workaround" verhindert Versagen der Anwendung bei abgeschalteter Security
    if (getUserID() == null)
    {
        setUserID("anonym"); 
      }
       buf.append(getUserID());
      buf.append(";");
      buf.append(getSessionID());
    setClientContextLogField(buf.toString());
  }

  /**
   * Gets the sessionID
   * @return Returns a String
   */
  public String getSessionID()
  {
    return sessionID;
  }
  /**
   * Sets the sessionID
   * @param sessionID The sessionID to set
   */
  public void setSessionID(String sessionID)
  {
    this.sessionID = sessionID;
    updateRecord();
  }

  /**
   * Gets the userID
   * @return Returns a String
   */
  public String getUserID()
  {
    return userID;
  }
  /**
   * Gets the userName
   * @return Returns a String
   * @deprecated
   */
  public String getUserName()
  {
    return getUserID();
  }

  /**
   * Sets the userID
   * @param userID The userID to set
   */
  public void setUserID(String userID)
  {
    this.userID = userID;
    updateRecord();
  }

  /**
   * Sets the userName
   * @param userName The userName to set
   * @deprecated
   */
  public void setUserName(String userID)
  {
    setUserID(userID);
    updateRecord();
  }

  public Object clone()
  {
    Object theClone = null;
    theClone = super.clone();
    return theClone;
  }

}